@@ -58,8 +58,6 @@ static DBusMessage *get_properties(DBusConnection *conn,
 			DBUS_TYPE_OBJECT_PATH, __connman_profile_list, NULL);
 	connman_dbus_dict_append_array(&dict, "Services",
 			DBUS_TYPE_OBJECT_PATH, __connman_service_list, NULL);
-	connman_dbus_dict_append_array(&dict, "Providers",
-			DBUS_TYPE_OBJECT_PATH, __connman_provider_list, NULL);
 	connman_dbus_dict_append_array(&dict, "Technologies",
 			DBUS_TYPE_OBJECT_PATH, __connman_technology_list, NULL);
 
@@ -127,7 +125,7 @@ static DBusMessage *set_property(DBusConnection *conn,
 
 		dbus_message_iter_get_basic(&value, &offlinemode);
 
-		__connman_profile_set_offlinemode(offlinemode);
+		__connman_profile_set_offlinemode(offlinemode, TRUE);
 
 		__connman_profile_save_default();
 	} else if (g_str_equal(name, "Tethering") == TRUE) {
